unless defined?(Motion::Project::Config)
  raise 'This file must be required within a RubyMotion project Rakefile.'
end

class ConverserConfig
  attr_accessor :framework, :server_url, :app_api_key

  def initialize(config)
    @config = config
  end

  def framework=(path)
    if @framework != path
      @config.unvendor_project(@framework)
      @framework = path
      @config.vendor_project(path, :static, :products => ['libVGConversationKit_universal.a'], :headers_dir => 'include')
      @config.resources_dirs << "#{path}/ConverserResources"
      @config.frameworks << 'MessageUI'
      create_code
    end
  end

  def app_api_key=(app_api_key)
    @app_api_key = app_api_key
    create_code
  end

  def server_url=(server_url)
    @server_url = server_url
    create_code
  end

  private

  def create_code
    return unless @framework && @app_api_key && @server_url
    code = <<EOF
# This file is automatically generated. Do not edit.

class ConverserEngine
  def self.instance
    Dispatch.once { @instance ||= VGConversationEngine.alloc.initWithHostName('#{@server_url}', andKey: '#{@app_api_key}') }
    @instance
  end
end
EOF
    converser_file = './app/converser_code.rb'
    create_stub(converser_file, code)
    add_file(converser_file)
  end

  def create_stub(path, code)
    if !File.exist?(path) or File.read(path) != code
      File.open(path, 'w') { |io| io.write(code) }
    end
  end

  def add_file(path)
    files = @config.files.flatten
    @config.files << path unless files.find { |x| File.expand_path(x) == File.expand_path(path) }
  end
end

module Motion
  module Project
    class Config
      variable :converser
      def converser
        @converser ||= ConverserConfig.new(self)
      end
    end
  end
end
