class ConverserViewController < UIViewController
  def init
    super.tap do
      self.converser = ConverserEngine.instance
    end
  end

  def loadView
    self.view = ConverserView.new
  end

  def viewDidLoad
    view.check_button.addTarget(
      self,
      action: :check_button_tapped,
      forControlEvents: UIControlEventTouchUpInside
    )

    view.check_button.addTarget(
      self,
      action: :read_button_tapped,
      forControlEvents: UIControlEventTouchUpInside
    )

    subscribe_to_converser
  end

  private

  attr_accessor :converser, :current_conversation

  def check_button_tapped
    check_for_conversations
  end

  def read_button_tapped
    read_conversation
  end

  def subscribe_to_converser
    converser.subscribeWithIdentity(
      'test@example.com',
      pushToken: nil,
      completion: lambda do |data|
        p 'Subscription successful'
      end,
      errorHandler: lambda do |error|
        p 'Subscription error'
      end
    )
  end

  def check_for_conversations
    converser.retrieveInboxWithCompletionBlock(
      lambda do |inboxItems|
        if inboxItems.count == 0
          p 'You have no conversations'
        else
          self.current_conversation = inboxItems[0]
          if current_conversation.status == 'unread'
            p 'You have a conversation!'
          else
            p 'Finished with conversation'
            finished_with_conversation(current_conversation)
          end
        end
      end,
      errorHandler: lambda do |error|
        p 'Server not available'
      end
    )
  end

  def read_conversation
    return unless current_conversation

    # Ask the Converser SDK to create a controller that will run the conversation, and display that modally.
    converserController = converser.viewControllerForConversation(
      current_conversation.conversationTrackerId,
      withDelegate: self
    )

    presentModalViewController(converserController, animated: true)
  end

  def finished_with_conversation(conversation)
    if conversation
      converser.removeInboxItem(
        conversation.reference,
        withCompletionBlock: lambda do |data|
          p 'Successfully finished conversatsion'
        end,
        errorHandler: lambda do |error|
          p 'Failed to finish conversation'
        end
      )
    else
      p 'Attempt to finish nil conversation'
    end
  end
end
